<?php
/**
 * $Horde: nic/lib/Device.php,v 1.9 2005/12/01 09:57:49 selsky Exp $
 *
 * @package Nic
 */
class Nic_Device {

    /**
     * Device parameters.
     *
     * @var array
     */
    var $params;

    /**
     * Returns the current status of the device.
     *
     * @return constant    The status of the device (NIC_STATUS_* constant)
     */
    function getStatus()
    {
        $status = array();
        $status['status'] = NIC_STATUS_UNKNOWN;
        $status['message'] = '';

        return $status;
    }

    /**
     * Rander the current details of the device in an HTML table.
     *
     * @param string    $templagePath  Path to the templates to use.
     * @param string    $deviceid      ID of the device, if set is use to create links
     */
    function showDetails($templatePath, $deviceid = false)
    {
    }

    /**
     * Rander the current status of the device in an HTML table.
     *
     * @param string    $templagePath  Path to the templates to use.
     * @param string    $deviceid      ID of the device, if set is use to create links
     */
    function showStatus($templatePath, $deviceid = false)
    {
        $name = $this->getName();
        $status = $this->getStatus();
        require $templatePath . '/header.inc';
        require $templatePath . '/footer.inc';
    }

    /**
     * Retrieve the current details of the device that can be stored
     * in a storage backend.
     *
     * @return    array      KEY: description of detail VALUE: value of that details
     */
    function retrieveStoreableData()
    {
        return array();
    }

    /**
     * Retrieve the details to draw the graphs for this device
     *
     * @return    array      KEY: description of detail VALUE: value of that details
     */
    function retrieveGraphDetails()
    {
        return $this->graphs;
    }

    /**
     * Attempts to return a concrete Nic_Devicer instance based on $driver.
     *
     * @param string    $driver     The type of concrete Nic_Device subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param string    $user       The name of the user who owns these tasks.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The newly created concrete Nic_Device instance, or
     *                  false on an error.
     */
    function &factory($driver, $params = array())
    {
        $driver = basename($driver);
        include_once dirname(__FILE__) . '/Device/' . $driver . '.php';
        $class = 'Nic_Device_' . $driver;
        if (class_exists($class)) {
            $device = &new $class($params);
        } else {
            $device = false;
        }

        return $device;
    }

    /**
     * Attempts to return a reference to a concrete Nic_Device instance based
     * on $driver.
     *
     * It will only create a new instance if no Nic_Device instance with the
     * same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Nic_Device::singleton()
     *
     * @param string    $driver     The type of concrete Nic_Device subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Nic_Device instance, or false
     *                  on error.
     */
    function &singleton($driver, $params = array())
    {
        static $instances;

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = md5(serialize(array($driver, $params)));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Nic_Device::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
