<?php

require_once NIC_BASE . '/lib/Device/SNMP.php';

/**
 * Nic_Device_SNMPCustom:: Nic_Device implementation to retrieve data
 * from any SNMP device.
 *
 * @since   Network 0.1
 * @package Nic
 */
class Nic_Device_SNMPCustom extends Nic_Device_SNMP {

    /**
     * SNMP Object ID's to retrieve.
     *
     * @var array
     */
    var $oids = array(
        array('name' => 'Uptime', 'oid' => '.1.3.6.1.2.1.1.3.0', 'storeable' => false),
        array('name' => 'System Description', 'oid' => '.1.3.6.1.2.1.1.1.0', 'storeable' => false),
    );

    /**
     * Array containing the details of graphs available for this device.
     *
     * @var array
     */
    var $_graphs = array();

    function Nic_Device_SNMPCustom($params = array())
    {
        $this->params = $params;
        if (!isset($this->params['timeout'])) {
            $this->params['timeout'] = 10;
        }

        // Load any custom oids.
        $this->oids = array_merge($this->oids, $params['oids']);
    }

    function getStatus()
    {
        $status = array();
        $status['status'] = NIC_STATUS_NORMAL;
        $status['message'] = '';

        $uptime = @snmpget($this->params['hostspec'] . ':' . $this->params['port'], $this->params['community'], '.1.3.6.1.2.1.1.3.0', $this->params['timeout'], 2);

        if ($uptime === false) {
            $status['status'] = NIC_STATUS_FAULT;
        } else {
            $status['message'] = sprintf(_("Uptime: %s"), $this->_cleanResponse($uptime));
        }

        return $status;
    }

    /**
     * Render the current details of the device in an HTML table
     *
     * @param string $templagePath  Path to the templates to use.
     * @param string $deviceid      ID of the device, if set is use to create links
     */
    function showDetails($templatePath, $device_id)
    {
        global $devices, $rrdtools;
        require $templatePath . '/header.inc';

        $details = $this->retrieveStoreableData();

        foreach ($details as $description => $values) {
            $value = $values['value'];
            require $templatePath . '/row.inc';
        }

        $rrdfiles = array();
        // Cycle through OIDs to find the filenames to create.
        foreach ($devices[$device_id]['params']['oids'] as $oid) {
            if (array_key_exists('rrdtool', $oid)) {
                $rrd = $oid['rrdtool'];
                $key = $rrdtools[$rrd]['file'];

                if (!array_key_exists($key, $rrdfiles)) {
                    $rrdfiles[$key] = array();
                }

                $rrdfiles[$key][] = $rrd;
            }
        }

        foreach ($rrdfiles as $rrdfile => $rrds) {
            require $templatePath . '/rrd-graph.inc';
        }

        require $templatePath . '/footer.inc';
    }

}
