<?php
/**
 * Nic_Device_imap:: Nic_Device implementation to retrieve data from
 * an IMAP server.
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @since   Nic 0.2
 * @package Nic
 */
class Nic_Device_imap extends Nic_Device {

    function Nic_Device_imap($params = array())
    {
        $this->params = $params;
        $this->params['hostspec'] = isset($this->params['hostspec']) ? $this->params['hostspec'] : 'localhost';
        $this->params['port'] = isset($this->params['port']) ? $this->params['port'] : 143;
    }

    function getStatus()
    {
        $status = array();
        $status['status'] = NIC_STATUS_NORMAL;
        $status['message'] = '';

        $hosts = explode(',', $this->params['hostspec']);
        foreach ($hosts as $host) {
            $result = $this->_getServerResponse(trim($host), $this->params['port']);

            $status['message'] .= 'Host: ' . $host . "\n";

            if ($result == false || is_a($result, 'PEAR_Error')) {
                $status['message'] .= _("Conenction Failed");
                $status['status'] = max($status['status'], NIC_STATUS_FAULT);
            } else {
                $status['message'] .= $result;
                if (!empty($result) && strlen($result) > 3 && substr($result, 0, 4) == "* OK") {
                    $status['status'] = max($status['status'], NIC_STATUS_NORMAL);
                } else {
                    $status['status'] = max($status['status'], NIC_STATUS_WARN);
                }
            }
            $status['message'] .= "\n\n";
        }

        return $status;
    }

    function _getServerResponse($host, $port)
    {
        require_once 'Net/Socket.php';
        $socket = &new Net_Socket();

        if (!@$socket->connect($host, $port, false, 3)) {
            return false;
        } else {
            $result = $socket->readLine();
            $socket->disconnect();
            return $result;
        }
    }

}
