<?php
/**
 * Nic_Storage:: defines an API for implementing storage backends for Network.
 *
 * $Horde: nic/lib/Storage.php,v 1.9 2005/12/01 09:57:50 selsky Exp $
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @since   Network 0.1
 * @package Nic
 */
class Nic_Storage {

    /**
     * String containing the current device ID.
     *
     * @var string
     */
    var $_device = '';

    /**
     * Attempts to return a concrete Nic_Storage instance based on $driver.
     *
     * @param string    $driver     The type of concrete Nic_Storage subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param string    $device     The name of the device that data relates to
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The newly created concrete Nic_Storage instance, or
     *                  false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        include_once dirname(__FILE__) . '/Storage/' . $driver . '.php';
        $class = 'Nic_Storage_' . $driver;
        if (class_exists($class)) {
            $storage = &new $class($params);
        } else {
            $storage = false;
        }

        return $storage;
    }

    /**
     * Attempts to return a reference to a concrete Nic_Storage instance
     * based on $driver.  It will only create a new instance if no
     * Nic_Storage instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Nic_Storage::singleton()
     *
     * @param string    $driver     The type of concrete Nic_Storage subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param string    $device     The name of the device that data relates to
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The concrete Nic_Storage instance, or
     *                  false on an error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Nic_Storage::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
