<?php
/**
 * $Horde: nic/rrdgraph.php,v 1.4 2007/01/02 00:41:47 jan Exp $
 *
 * Copyright 2005-2006 Mike Bydalek <mike@distance.net>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('NIC_BASE', dirname(__FILE__));
require_once NIC_BASE . '/lib/base.php';
require_once NIC_BASE . '/lib/Device.php';
require_once NIC_BASE . '/lib/Storage.php';

// Retrieve the form values.
$rrds = Util::getFormData('rrds');
$rrdfile = Util::getFormData('rrdfile');

// Since we need seconds ago, convert to a negative value.  Also
// default to 1 day ago until 5 mins ago.
$start_time = Util::getFormData('start', 24 * 3600) * -1;
$end_time = Util::getFormData('end', 301 ) * -1;
$step = Util::getFormData('step', 300);
$title = Util::getFormData('title');

global $conf, $rrdtools;

// Get data values.
$storage = &Nic_Storage::singleton('rrd');
$rrdfile = $storage->getFileName($rrdfile);

// Set default title of the rrdfilename
$title = $title?$title:$rrdtools[$rrds[0]]['file'];

// TODO: Get graph preferences.
// X-Grid, title?, width, height, X/Y Labels

$rrdcmd = $conf['rrdtool']['binary'] . " graph - ";
$rrdcmd .= "-s " . $start_time . " -e " . $end_time;
$rrdcmd .= " -a PNG --lazy";
$rrdcmd .= " --color FRAME#000000";
$rrdcmd .= " --title '" . $title . "'";
$rrdcmd .= " --width 500 --height 150";

// Do DEF: for each data source.
foreach ($rrds as $rrd) {
    $rrdcmd .= " DEF:" . $rrd . "=" . $rrdfile . ":" . $rrd . ":AVERAGE";
    $rrdcmd .= " DEF:" . $rrd . "_last=" . $rrdfile . ":" . $rrd . ":LAST";
    $rrdcmd .= " DEF:" . $rrd . "_max=" . $rrdfile . ":" . $rrd . ":MAX";
    $rrdcmd .= " DEF:" . $rrd . "_min=" . $rrdfile . ":" . $rrd . ":MIN";
}

// Do any calculations for data sources that require it.
foreach ($rrds as $rrd) {
    if (isset($rrdtools[$rrd]['graph_calc'])) {
        $rrd_new = $rrd . "_graph";
        $calc = "," . $rrdtools[$rrd]['graph_calc'];
        $rrdcmd .= " CDEF:" . $rrd_new . "=" . $rrd . $calc;
        $rrdcmd .= " CDEF:" . $rrd_new . "_last=" . $rrd . "_last" . $calc;
        $rrdcmd .= " CDEF:" . $rrd_new . "_min=" . $rrd . "_min" . $calc;
        $rrdcmd .= " CDEF:" . $rrd_new . "_max=" . $rrd . "_max" . $calc;
    }
    if (isset($rrdtools[$rrd]['text_calc'])) {
        $rrd_new = $rrd . "_text";
        $calc = "," . $rrdtools[$rrd]['text_calc'];
        $rrdcmd .= " CDEF:" . $rrd_new . "=" . $rrd . $calc;
        $rrdcmd .= " CDEF:" . $rrd_new . "_last=" . $rrd . "_last" . $calc;
        $rrdcmd .= " CDEF:" . $rrd_new . "_min=" . $rrd . "_min" . $calc;
        $rrdcmd .= " CDEF:" . $rrd_new . "_max=" . $rrd . "_max" . $calc;
    }
}

// Do line types on actual graph for each data source.
foreach ($rrds as $rrd) {
    if (isset($rrdtools[$rrd]['graph_calc'])) {
        $rrd_new = $rrd . "_graph";
    } else {
        $rrd_new = $rrd;
    }

    $rrdcmd .= " " . $rrdtools[$rrd]['line'] . ":" . $rrd_new . "_last" . $rrdtools[$rrd]['color'] . ":" . $rrdtools[$rrd]['name'];

    // Check if we want an outline of the data on our graph.
    if (isset($rrdtools[$rrd]['outline']) && $rrdtools[$rrd]['outline']) {
        $rrdcmd .= ' LINE1:' . $rrd_new . '_last#336600';
    }
}

$rrdcmd .= " COMMENT:'\\n'";

$graph = array();

// Do graph labels.
foreach ($rrds as $rrd) {
    if (isset($rrdtools[$rrd]['text_calc'])) {
        $rrd_new = $rrd . '_text';
    } else {
        $rrd_new = $rrd;
    }

    $graph[] = 'GPRINT:' . $rrd_new . "_last:LAST:'Current - %.2lf " . $rrdtools[$rrd]['unit'] . "'";
    $graph[] = 'GPRINT:' . $rrd_new . ":AVERAGE:'Average - %.2lf " . $rrdtools[$rrd]['unit'] . "'";
    $graph[] = 'GPRINT:' . $rrd_new . "_max:MAX:'Maximum - %.2lf " . $rrdtools[$rrd]['unit'] . "'";
    $graph[] = 'GPRINT:' . $rrd_new . "_min:MIN:'Minimum - %.2lf " . $rrdtools[$rrd]['unit'] . "'";
}

// Format the labels nicely in a per column, rather than per-row.
$step = count($graph) / count($rrds);
for ($i = 0; $i < $step; $i++) {
    for ($j = $i; $j < count($graph); $j += $step) {
        $rrdcmd .= ' ' . $graph[$j];
    }
    $rrdcmd .= " COMMENT:'\\n'";
}

// Make sure the image is not cached.
header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');              // Date in the past.
header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT'); // Always modified.
header('Cache-Control: no-cache, must-revalidate');            // HTTP/1.1
header('Pragma: no-cache');                                    // HTTP/1.0
header('Content-type: image/png');
passthru($rrdcmd);
