#!/usr/local/bin/php -q
<?php
/**
 * $Horde: nic/scripts/createRRDFiles.php,v 1.3 2006/04/18 15:39:36 jan Exp $
 */

@define('AUTH_HANDLER', true);
@define('NIC_BASE', dirname(__FILE__) . '/..');
@define('HORDE_BASE', dirname(__FILE__) . '/../..');

require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// This shouldn't be callable via the web.
if (!Horde_CLI::runningFromCLI()) {
    echo _("This script must be run from the command line.") . "\n";
    exit;
} else {
    Horde_CLI::init();
    $cli = &Horde_CLI::singleton();
}

require_once NIC_BASE . '/lib/base.php';
require_once NIC_BASE . '/lib/Storage.php';

global $rrdtools;

$rrd_storage = &Nic_Storage::singleton("rrd");

$rras = array("AVERAGE", "LAST", "MAX", "MIN");
$sources = array ("GAUGE", "COUNTER", "DERIVE", "ABSOLUTE");

$rrdfiles = array();
foreach ($devices as $deviceid => $device) {
    if (!array_key_exists('oids', $device['params'])) {
        break;
    }

    // Cycle through OIDs to find the filenames to create.
    foreach ($device['params']['oids'] as $oid) {
        if (array_key_exists('rrdtool', $oid)) {
            $rrd = $oid['rrdtool'];
            $key = $rrd_storage->getFileName($rrdtools[$rrd]['file']);

            if (!array_key_exists($key, $rrdfiles)) {
                $rrdfiles[$key] = array();
            }

            $rrdfiles[$key][] = $rrd;
        }
    }
}

$cli->writeln("This script has detected the following RRD Files:");
foreach ($rrdfiles as $rrdfile => $rrds) {
    if (!file_exists($rrdfile)) {
        $cli->writeln($cli->indent($cli->yellow("NEW: " . $rrdfile)));
    } else {
        $cli->writeln($cli->indent($cli->green("NOT CREATING: " . $rrdfile)));
    }
}

$result = $cli->prompt("Continue?", array("No", "Yes"));

if (!$result) {
    $cli->writeln($cli->red("Operation aborted."));
    exit;
}

$seconds = '';
while (!is_numeric($seconds)) {
    $seconds = $cli->prompt("How often are you going to run the statusMonitor.php script (in seconds)?");
}

// TODO: Explain.
$archives = array();

foreach ($rrdfiles as $rrdfile => $rrds) {
    if (file_exists($rrdfile)) {
        continue;
    }

    $cli->writeln($cli->indent($cli->yellow("Processing file: " . $rrdfile)));

    // Choose type, counter, etc.
    $types = array($cli->bold("GAUGE") . " is for things like temperatures or number of people in a room or value of a RedHat share.",
                   $cli->bold("COUNTER") . " is for continuous incrementing counters like the InOctets counter in a router.",
                   $cli->bold("DERIVE") . " will store the derivative of the line going from the last to the current value of the data source.",
                   $cli->bold("ABSOLUTE") . " is for counters which get reset upon reading.");
    $ds = $sources[$cli->prompt("What type of Data Source should be used?", $types)];

    $finished = false;

    // Check if to use new or same view...
    if (sizeof($archives) > 0) {
        $keep = $cli->prompt("Keep the same archives as previous RRD File?", array("No", "Yes"));

        if (!$keep) {
            $archives = array();
        }
    }

    while (!$finished) {
        // Show what archives we have.
        $cli->writeln($cli->green("Current archives:"));
        foreach ($archives as $archive) {
            $cli->writeln($cli->indent($cli->green(format_time($archive[1]) . " of data averaged every " . format_time($archive[0]))));
        }

        $choice = $cli->prompt("Choice?", array("New Archive", "Delete Archive", "Create Defaults", "Finish"));

        switch ($choice) {
        case '0': // Create new archive
            $average = '';
            while (!is_numeric($average)) {
                $average = $cli->prompt("How many seconds do you want to average the data (in seconds)?");
            }
            $points = '';
            while (!is_numeric($points)) {
                $points = $cli->prompt("For how long to do you want to keep these averages (in seconds)?");
            }
            $archives[] = array($average, $points);
            break;

        case '1': // Delete an archive
            $options = array();
            foreach ($archives as $archive) {
                $options[] = format_time($archive[1]) . " of data averaged every " . format_time($archive[0]);
            }
            $result = $cli->prompt("Select archive to delete:", $options);

            unset($archives[$result]);
            array_merge($archives);
            break;

        case '2': // Create default archives.
            $archives = array(array($seconds, 86400),   // $seconds averages over 1 day
                              array(1800, 604800),      // 30 min averages over 1 week
                              array(7200, 2592000),     // 2 hour averages over 1 month
                              array(86400, 31556926));  // 1 day averages over 1 year
            break;

        case '3': // Create RRD file.
            if (sizeof($archives) == 0) {
                $cli->writeln($cli->red("You must create some archives first!"));
                break;
            }
            $rrdcreate = $conf['rrdtool']['binary'] . " create " . $rrdfile . " --step " . $seconds;
            foreach ($rrds as $rrd) {
                /* We have to replace anything that's not [a-zA-Z0-9].
                   See 'man rrdcreate' for more details. */
                // TODO: Error out if not valid $rrd.
                $rrdcreate .= ' DS:' . eregi_replace('[^a-zA-Z0-9_]', '_', $rrd) . ':' . $ds . ':' . round($seconds * 1.5) . ':U:U';
            }

            foreach ($archives as $archive) {
                foreach ($rras as $rra) {
                    $rrdcreate .= ' RRA:' . $rra . ':0.5:' . round($archive[0] / $seconds) . ':' . round($archive[1] / $archive[0]);
                }
            }

            $finished = true;
            $res = system($rrdcreate);
            $cli->writeln("Please remember to change the permissions on the file '" . $rrdfile . "' to allow your webserver to read the file.");
            break;
        }
    }
}

function format_time($seconds)
{
    $time = '';
    if ($seconds >= 31556926) {
        $years = floor($seconds/31556926);
        $seconds = $seconds % 31556926;
        $time .= $years . ' years ';
    }
    if ($seconds >= 2592000) {
        $months = floor($seconds/2592000);
        $seconds = $seconds % 2592000;
        $time .= $months . ' months ';
    }
    if ($seconds >= 86400) {
        $days = floor($seconds/86400);
        $seconds = $seconds % 86400;
        $time .= $days . ' days ';
    }
    if ($seconds >= 3600) {
        $hours = floor($seconds/3600);
        $seconds = $seconds % 3600;
        $time .= $hours . ' hours ';
    }
    if ($seconds >= 60) {
        $mins = floor($seconds/60);
        $seconds = $seconds % 60;
        $time .= $mins . ' minutes ';
    }
    if ($seconds > 0) {
        $time .= $seconds . ' seconds ';
    }

    return substr($time, 0, -1);
}
